@@ -1,4 +1,8 @@
-use std::{env, path::Path, str::FromStr};
+use std::{
+    env,
+    path::{Path, PathBuf},
+    str::FromStr,
+};
 
 use async_trait::async_trait;
 use atuin_common::utils;
@@ -6,6 +10,7 @@ use chrono::{prelude::*, Utc};
 use fs_err as fs;
 use itertools::Itertools;
 use lazy_static::lazy_static;
+use rand::{distributions::Alphanumeric, Rng};
 use regex::Regex;
 use sql_builder::{esc, quote, SqlBuilder, SqlName};
 use sqlx::{
@@ -16,13 +21,15 @@ use sqlx::{
 use super::{
     history::History,
     ordering,
-    settings::{FilterMode, SearchMode},
+    settings::{FilterMode, SearchMode, Settings},
 };
 
 pub struct Context {
     pub session: String,
     pub cwd: String,
     pub hostname: String,
+    pub host_id: String,
+    pub git_root: Option<PathBuf>,
 }
 
 #[derive(Default, Clone)]
@@ -43,13 +50,21 @@ pub fn current_context() -> Context {
         eprintln!("ERROR: Failed to find $ATUIN_SESSION in the environment. Check that you have correctly set up your shell.");
         std::process::exit(1);
     };
-    let hostname = format!("{}:{}", whoami::hostname(), whoami::username());
+    let hostname = format!(
+        "{}:{}",
+        env::var("ATUIN_HOST_NAME").unwrap_or_else(|_| whoami::hostname()),
+        env::var("ATUIN_HOST_USER").unwrap_or_else(|_| whoami::username())
+    );
     let cwd = utils::get_current_dir();
+    let host_id = Settings::host_id().expect("failed to load host ID");
+    let git_root = utils::in_git_repo(cwd.as_str());
 
     Context {
         session,
         hostname,
         cwd,
+        git_root,
+        host_id: host_id.0.as_simple().to_string(),
     }
 }
 
@@ -160,17 +175,18 @@ impl Sqlite {
     fn query_history(row: SqliteRow) -> History {
         let deleted_at: Option<i64> = row.get("deleted_at");
 
-        History {
-            id: row.get("id"),
-            timestamp: Utc.timestamp_nanos(row.get("timestamp")),
-            duration: row.get("duration"),
-            exit: row.get("exit"),
-            command: row.get("command"),
-            cwd: row.get("cwd"),
-            session: row.get("session"),
-            hostname: row.get("hostname"),
-            deleted_at: deleted_at.map(|t| Utc.timestamp_nanos(t)),
-        }
+        History::from_db()
+            .id(row.get("id"))
+            .timestamp(Utc.timestamp_nanos(row.get("timestamp")))
+            .duration(row.get("duration"))
+            .exit(row.get("exit"))
+            .command(row.get("command"))
+            .cwd(row.get("cwd"))
+            .session(row.get("session"))
+            .hostname(row.get("hostname"))
+            .deleted_at(deleted_at.map(|t| Utc.timestamp_nanos(t)))
+            .build()
+            .into()
     }
 }
 
@@ -252,6 +268,7 @@ impl Database for Sqlite {
             FilterMode::Host => query.and_where_eq("hostname", quote(&context.hostname)),
             FilterMode::Session => query.and_where_eq("session", quote(&context.session)),
             FilterMode::Directory => query.and_where_eq("cwd", quote(&context.cwd)),
+            FilterMode::Workspace => query.and_where_like_any("cwd", context.cwd.clone()),
         };
 
         if unique {
@@ -371,11 +388,18 @@ impl Database for Sqlite {
             sql.order_desc("timestamp");
         }
 
+        let git_root = if let Some(git_root) = context.git_root.clone() {
+            git_root.to_str().unwrap_or("/").to_string()
+        } else {
+            context.cwd.clone()
+        };
+
         match filter {
             FilterMode::Global => &mut sql,
             FilterMode::Host => sql.and_where_eq("hostname", quote(&context.hostname)),
             FilterMode::Session => sql.and_where_eq("session", quote(&context.session)),
             FilterMode::Directory => sql.and_where_eq("cwd", quote(&context.cwd)),
+            FilterMode::Workspace => sql.and_where_like_left("cwd", git_root),
         };
 
         let orig_query = query;
@@ -456,6 +480,8 @@ impl Database for Sqlite {
                 .map(|after| sql.and_where_gt("timestamp", quote(after.timestamp_nanos())))
         });
 
+        sql.and_where_is_null("deleted_at");
+
         let query = sql.sql().expect("bug in search query. please report");
 
         let res = sqlx::query(&query)
@@ -515,7 +541,11 @@ impl Database for Sqlite {
     // but the time that the system marks it as deleted
     async fn delete(&self, mut h: History) -> Result<()> {
         let now = chrono::Utc::now();
-        h.command = String::from(""); // blank it
+        h.command = rand::thread_rng()
+            .sample_iter(&Alphanumeric)
+            .take(32)
+            .map(char::from)
+            .collect(); // overwrite with random string
         h.deleted_at = Some(now); // delete it
 
         self.update(&h).await?; // save it
@@ -540,6 +570,8 @@ mod test {
             hostname: "test:host".to_string(),
             session: "beepboopiamasession".to_string(),
             cwd: "/home/ellie".to_string(),
+            host_id: "test-host".to_string(),
+            git_root: None,
         };
 
         let results = db
@@ -579,17 +611,19 @@ mod test {
     }
 
     async fn new_history_item(db: &mut impl Database, cmd: &str) -> Result<()> {
-        let history = History::new(
-            chrono::Utc::now(),
-            cmd.to_string(),
-            "/home/ellie".to_string(),
-            0,
-            1,
-            Some("beep boop".to_string()),
-            Some("booop".to_string()),
-            None,
-        );
-        db.save(&history).await
+        let mut captured: History = History::capture()
+            .timestamp(chrono::Utc::now())
+            .command(cmd)
+            .cwd("/home/ellie")
+            .build()
+            .into();
+
+        captured.exit = 0;
+        captured.duration = 1;
+        captured.session = "beep boop".to_string();
+        captured.hostname = "booop".to_string();
+
+        db.save(&captured).await
     }
 
     #[tokio::test(flavor = "multi_thread")]
@@ -746,6 +780,8 @@ mod test {
             hostname: "test:host".to_string(),
             session: "beepboopiamasession".to_string(),
             cwd: "/home/ellie".to_string(),
+            host_id: "test-host".to_string(),
+            git_root: None,
         };
 
         let mut db = Sqlite::new("sqlite::memory:").await.unwrap();
