@@ -11,8 +11,9 @@ pub mod encryption;
 pub mod sync;
 
 pub mod database;
-pub mod event;
 pub mod history;
 pub mod import;
+pub mod kv;
 pub mod ordering;
+pub mod record;
 pub mod settings;
