@@ -1,8 +1,10 @@
 use std::{
     io::prelude::*,
     path::{Path, PathBuf},
+    str::FromStr,
 };
 
+use atuin_common::record::HostId;
 use chrono::{prelude::*, Utc};
 use clap::ValueEnum;
 use config::{Config, Environment, File as ConfigFile, FileFormat};
@@ -12,11 +14,13 @@ use parse_duration::parse;
 use regex::RegexSet;
 use semver::Version;
 use serde::Deserialize;
+use uuid::Uuid;
 
 pub const HISTORY_PAGE_SIZE: i64 = 100;
 pub const LAST_SYNC_FILENAME: &str = "last_sync_time";
 pub const LAST_VERSION_CHECK_FILENAME: &str = "last_version_check_time";
 pub const LATEST_VERSION_FILENAME: &str = "latest_version";
+pub const HOST_ID_FILENAME: &str = "host_id";
 
 #[derive(Clone, Debug, Deserialize, Copy, ValueEnum, PartialEq)]
 pub enum SearchMode {
@@ -68,6 +72,9 @@ pub enum FilterMode {
 
     #[serde(rename = "directory")]
     Directory = 3,
+
+    #[serde(rename = "workspace")]
+    Workspace = 4,
 }
 
 impl FilterMode {
@@ -77,6 +84,7 @@ impl FilterMode {
             FilterMode::Host => "HOST",
             FilterMode::Session => "SESSION",
             FilterMode::Directory => "DIRECTORY",
+            FilterMode::Workspace => "WORKSPACE",
         }
     }
 }
@@ -140,6 +148,7 @@ pub struct Settings {
     pub sync_address: String,
     pub sync_frequency: String,
     pub db_path: String,
+    pub record_store_path: String,
     pub key_path: String,
     pub session_path: String,
     pub search_mode: SearchMode,
@@ -149,6 +158,7 @@ pub struct Settings {
     pub inline_height: u16,
     pub invert: bool,
     pub show_preview: bool,
+    pub max_preview_height: u16,
     pub show_help: bool,
     pub exit_mode: ExitMode,
     pub word_jump_mode: WordJumpMode,
@@ -158,6 +168,8 @@ pub struct Settings {
     pub history_filter: RegexSet,
     #[serde(with = "serde_regex", default = "RegexSet::empty")]
     pub cwd_filter: RegexSet,
+    pub workspaces: bool,
+    pub ctrl_n_shortcuts: bool,
 
     // This is automatically loaded when settings is created. Do not set in
     // config! Keep secrets and settings apart.
@@ -226,6 +238,23 @@ impl Settings {
         Settings::load_time_from_file(LAST_VERSION_CHECK_FILENAME)
     }
 
+    pub fn host_id() -> Option<HostId> {
+        let id = Settings::read_from_data_dir(HOST_ID_FILENAME);
+
+        if let Some(id) = id {
+            let parsed =
+                Uuid::from_str(id.as_str()).expect("failed to parse host ID from local directory");
+            return Some(HostId(parsed));
+        }
+
+        let uuid = atuin_common::utils::uuid_v7();
+
+        Settings::save_to_data_dir(HOST_ID_FILENAME, uuid.as_simple().to_string().as_ref())
+            .expect("Could not write host ID to data dir");
+
+        Some(HostId(uuid))
+    }
+
     pub fn should_sync(&self) -> Result<bool> {
         if !self.auto_sync || !PathBuf::from(self.session_path.as_str()).exists() {
             return Ok(false);
@@ -321,23 +350,27 @@ impl Settings {
         config_file.push("config.toml");
 
         let db_path = data_dir.join("history.db");
+        let record_store_path = data_dir.join("records.db");
+
         let key_path = data_dir.join("key");
         let session_path = data_dir.join("session");
 
         let mut config_builder = Config::builder()
             .set_default("db_path", db_path.to_str())?
+            .set_default("record_store_path", record_store_path.to_str())?
             .set_default("key_path", key_path.to_str())?
             .set_default("session_path", session_path.to_str())?
             .set_default("dialect", "us")?
             .set_default("auto_sync", true)?
             .set_default("update_check", true)?
             .set_default("sync_address", "https://api.atuin.sh")?
-            .set_default("sync_frequency", "1h")?
+            .set_default("sync_frequency", "10m")?
             .set_default("search_mode", "fuzzy")?
             .set_default("filter_mode", "global")?
             .set_default("style", "auto")?
             .set_default("inline_height", 0)?
             .set_default("show_preview", false)?
+            .set_default("max_preview_height", 4)?
             .set_default("show_help", true)?
             .set_default("invert", false)?
             .set_default("exit_mode", "return-original")?
@@ -349,6 +382,8 @@ impl Settings {
             .set_default("scroll_context_lines", 1)?
             .set_default("shell_up_key_binding", false)?
             .set_default("session_token", "")?
+            .set_default("workspaces", false)?
+            .set_default("ctrl_n_shortcuts", false)?
             .add_source(
                 Environment::with_prefix("atuin")
                     .prefix_separator("_")
