@@ -10,11 +10,14 @@ use http::request::Parts;
 use tower::ServiceBuilder;
 use tower_http::trace::TraceLayer;
 
-use super::{database::Database, handlers};
-use crate::{models::User, settings::Settings};
+use super::handlers;
+use crate::settings::Settings;
+use atuin_server_database::{models::User, Database};
+
+pub struct UserAuth(pub User);
 
 #[async_trait]
-impl<DB: Send + Sync> FromRequestParts<AppState<DB>> for User
+impl<DB: Send + Sync> FromRequestParts<AppState<DB>> for UserAuth
 where
     DB: Database,
 {
@@ -45,24 +48,21 @@ where
             .await
             .map_err(|_| http::StatusCode::FORBIDDEN)?;
 
-        Ok(user)
+        Ok(UserAuth(user))
     }
 }
 
 async fn teapot() -> impl IntoResponse {
-    (http::StatusCode::IM_A_TEAPOT, "☕")
+    (http::StatusCode::IM_A_TEAPOT, "🫖")
 }
 
 #[derive(Clone)]
-pub struct AppState<DB> {
+pub struct AppState<DB: Database> {
     pub database: DB,
-    pub settings: Settings,
+    pub settings: Settings<DB::Settings>,
 }
 
-pub fn router<DB: Database + Clone + Send + Sync + 'static>(
-    database: DB,
-    settings: Settings,
-) -> Router {
+pub fn router<DB: Database>(database: DB, settings: Settings<DB::Settings>) -> Router {
     let routes = Router::new()
         .route("/", get(handlers::index))
         .route("/sync/count", get(handlers::history::count))
@@ -71,6 +71,9 @@ pub fn router<DB: Database + Clone + Send + Sync + 'static>(
         .route("/sync/status", get(handlers::status::status))
         .route("/history", post(handlers::history::add))
         .route("/history", delete(handlers::history::delete))
+        .route("/record", post(handlers::record::post))
+        .route("/record", get(handlers::record::index))
+        .route("/record/next", get(handlers::record::next))
         .route("/user/:username", get(handlers::user::get))
         .route("/account", delete(handlers::user::delete))
         .route("/register", post(handlers::user::register))
