@@ -23,15 +23,15 @@ use super::{
     engines::{SearchEngine, SearchState},
     history_list::{HistoryList, ListState, PREFIX_LENGTH},
 };
-use crate::ratatui::{
+use crate::{command::client::search::engines, VERSION};
+use ratatui::{
     backend::{Backend, CrosstermBackend},
     layout::{Alignment, Constraint, Direction, Layout},
     style::{Color, Modifier, Style},
-    text::{Span, Spans, Text},
+    text::{Line, Span, Text},
     widgets::{Block, BorderType, Borders, Paragraph},
     Frame, Terminal, TerminalOptions, Viewport,
 };
-use crate::{command::client::search::engines, VERSION};
 
 const RETURN_ORIGINAL: usize = usize::MAX;
 const RETURN_QUERY: usize = usize::MAX - 1;
@@ -42,6 +42,7 @@ struct State {
     results_state: ListState,
     switched_search_mode: bool,
     search_mode: SearchMode,
+    results_len: usize,
 
     search: SearchState,
     engine: Box<dyn SearchEngine>,
@@ -58,27 +59,26 @@ impl State {
     async fn query_results(&mut self, db: &mut dyn Database) -> Result<Vec<History>> {
         let results = self.engine.query(&self.search, db).await?;
         self.results_state.select(0);
+        self.results_len = results.len();
         Ok(results)
     }
 
-    fn handle_input(&mut self, settings: &Settings, input: &Event, len: usize) -> Option<usize> {
+    fn handle_input(&mut self, settings: &Settings, input: &Event) -> Option<usize> {
         match input {
-            Event::Key(k) => self.handle_key_input(settings, k, len),
-            Event::Mouse(m) => self.handle_mouse_input(*m, len),
+            Event::Key(k) => self.handle_key_input(settings, k),
+            Event::Mouse(m) => self.handle_mouse_input(*m),
             Event::Paste(d) => self.handle_paste_input(d),
             _ => None,
         }
     }
 
-    fn handle_mouse_input(&mut self, input: MouseEvent, len: usize) -> Option<usize> {
+    fn handle_mouse_input(&mut self, input: MouseEvent) -> Option<usize> {
         match input.kind {
             event::MouseEventKind::ScrollDown => {
-                let i = self.results_state.selected().saturating_sub(1);
-                self.results_state.select(i);
+                self.scroll_down(1);
             }
             event::MouseEventKind::ScrollUp => {
-                let i = self.results_state.selected() + 1;
-                self.results_state.select(i.min(len - 1));
+                self.scroll_up(1);
             }
             _ => {}
         }
@@ -94,12 +94,7 @@ impl State {
 
     #[allow(clippy::too_many_lines)]
     #[allow(clippy::cognitive_complexity)]
-    fn handle_key_input(
-        &mut self,
-        settings: &Settings,
-        input: &KeyEvent,
-        len: usize,
-    ) -> Option<usize> {
+    fn handle_key_input(&mut self, settings: &Settings, input: &KeyEvent) -> Option<usize> {
         if input.kind == event::KeyEventKind::Release {
             return None;
         }
@@ -107,10 +102,13 @@ impl State {
         let ctrl = input.modifiers.contains(KeyModifiers::CONTROL);
         let alt = input.modifiers.contains(KeyModifiers::ALT);
 
+        // Use Ctrl-n instead of Alt-n?
+        let modfr = if settings.ctrl_n_shortcuts { ctrl } else { alt };
+
         // reset the state, will be set to true later if user really did change it
         self.switched_search_mode = false;
         match input.code {
-            KeyCode::Char('c' | 'd' | 'g') if ctrl => return Some(RETURN_ORIGINAL),
+            KeyCode::Char('c' | 'g') if ctrl => return Some(RETURN_ORIGINAL),
             KeyCode::Esc => {
                 return Some(match settings.exit_mode {
                     ExitMode::ReturnOriginal => RETURN_ORIGINAL,
@@ -120,7 +118,7 @@ impl State {
             KeyCode::Enter => {
                 return Some(self.results_state.selected());
             }
-            KeyCode::Char(c @ '1'..='9') if alt => {
+            KeyCode::Char(c @ '1'..='9') if modfr => {
                 let c = c.to_digit(10)? as usize;
                 return Some(self.results_state.selected() + c);
             }
@@ -170,6 +168,12 @@ impl State {
             KeyCode::Delete => {
                 self.search.input.remove();
             }
+            KeyCode::Char('d') if ctrl => {
+                if self.search.input.as_str().is_empty() {
+                    return Some(RETURN_ORIGINAL);
+                }
+                self.search.input.remove();
+            }
             KeyCode::Char('w') if ctrl => {
                 // remove the first batch of whitespace
                 while matches!(self.search.input.back(), Some(c) if c.is_whitespace()) {}
@@ -183,15 +187,27 @@ impl State {
             }
             KeyCode::Char('u') if ctrl => self.search.input.clear(),
             KeyCode::Char('r') if ctrl => {
-                pub static FILTER_MODES: [FilterMode; 4] = [
-                    FilterMode::Global,
-                    FilterMode::Host,
-                    FilterMode::Session,
-                    FilterMode::Directory,
-                ];
+                let filter_modes = if settings.workspaces && self.search.context.git_root.is_some()
+                {
+                    vec![
+                        FilterMode::Global,
+                        FilterMode::Host,
+                        FilterMode::Session,
+                        FilterMode::Directory,
+                        FilterMode::Workspace,
+                    ]
+                } else {
+                    vec![
+                        FilterMode::Global,
+                        FilterMode::Host,
+                        FilterMode::Session,
+                        FilterMode::Directory,
+                    ]
+                };
+
                 let i = self.search.filter_mode as usize;
-                let i = (i + 1) % FILTER_MODES.len();
-                self.search.filter_mode = FILTER_MODES[i];
+                let i = (i + 1) % filter_modes.len();
+                self.search.filter_mode = filter_modes[i];
             }
             KeyCode::Char('s') if ctrl => {
                 self.switched_search_mode = true;
@@ -211,39 +227,45 @@ impl State {
                 })
             }
             KeyCode::Down if !settings.invert => {
-                let i = self.results_state.selected().saturating_sub(1);
-                self.results_state.select(i);
+                self.scroll_down(1);
             }
             KeyCode::Up if settings.invert => {
-                let i = self.results_state.selected().saturating_sub(1);
-                self.results_state.select(i);
+                self.scroll_down(1);
             }
-            KeyCode::Char('n' | 'j') if ctrl => {
-                let i = self.results_state.selected().saturating_sub(1);
-                self.results_state.select(i);
+            KeyCode::Char('n' | 'j') if ctrl && !settings.invert => {
+                self.scroll_down(1);
+            }
+            KeyCode::Char('n' | 'j') if ctrl && settings.invert => {
+                self.scroll_up(1);
             }
             KeyCode::Up if !settings.invert => {
-                let i = self.results_state.selected() + 1;
-                self.results_state.select(i.min(len - 1));
+                self.scroll_up(1);
             }
             KeyCode::Down if settings.invert => {
-                let i = self.results_state.selected() + 1;
-                self.results_state.select(i.min(len - 1));
+                self.scroll_up(1);
             }
-            KeyCode::Char('p' | 'k') if ctrl => {
-                let i = self.results_state.selected() + 1;
-                self.results_state.select(i.min(len - 1));
+            KeyCode::Char('p' | 'k') if ctrl && !settings.invert => {
+                self.scroll_up(1);
+            }
+            KeyCode::Char('p' | 'k') if ctrl && settings.invert => {
+                self.scroll_down(1);
             }
             KeyCode::Char(c) => self.search.input.insert(c),
-            KeyCode::PageDown => {
+            KeyCode::PageDown if !settings.invert => {
                 let scroll_len = self.results_state.max_entries() - settings.scroll_context_lines;
-                let i = self.results_state.selected().saturating_sub(scroll_len);
-                self.results_state.select(i);
+                self.scroll_down(scroll_len);
             }
-            KeyCode::PageUp => {
+            KeyCode::PageDown if settings.invert => {
                 let scroll_len = self.results_state.max_entries() - settings.scroll_context_lines;
-                let i = self.results_state.selected() + scroll_len;
-                self.results_state.select(i.min(len - 1));
+                self.scroll_up(scroll_len);
+            }
+            KeyCode::PageUp if !settings.invert => {
+                let scroll_len = self.results_state.max_entries() - settings.scroll_context_lines;
+                self.scroll_up(scroll_len);
+            }
+            KeyCode::PageUp if settings.invert => {
+                let scroll_len = self.results_state.max_entries() - settings.scroll_context_lines;
+                self.scroll_down(scroll_len);
             }
             _ => {}
         };
@@ -251,6 +273,16 @@ impl State {
         None
     }
 
+    fn scroll_down(&mut self, scroll_len: usize) {
+        let i = self.results_state.selected().saturating_sub(scroll_len);
+        self.results_state.select(i);
+    }
+
+    fn scroll_up(&mut self, scroll_len: usize) {
+        let i = self.results_state.selected() + scroll_len;
+        self.results_state.select(i.min(self.results_len - 1));
+    }
+
     #[allow(clippy::cast_possible_truncation)]
     #[allow(clippy::bool_to_int_with_if)]
     fn draw<T: Backend>(&mut self, f: &mut Frame<'_, T>, results: &[History], settings: &Settings) {
@@ -268,9 +300,14 @@ impl State {
                 .max_by(|h1, h2| h1.command.len().cmp(&h2.command.len()));
             longest_command.map_or(0, |v| {
                 std::cmp::min(
-                    4,
-                    (v.command.len() as u16 + preview_width - 1 - border_size)
-                        / (preview_width - border_size),
+                    settings.max_preview_height,
+                    v.command
+                        .split('\n')
+                        .map(|line| {
+                            (line.len() as u16 + preview_width - 1 - border_size)
+                                / (preview_width - border_size)
+                        })
+                        .sum(),
                 )
             }) + border_size * 2
         } else if compact {
@@ -373,7 +410,7 @@ impl State {
 
     #[allow(clippy::unused_self)]
     fn build_help(&mut self) -> Paragraph {
-        let help = Paragraph::new(Text::from(Spans::from(vec![
+        let help = Paragraph::new(Text::from(Line::from(vec![
             Span::styled("Esc", Style::default().add_modifier(Modifier::BOLD)),
             Span::raw(" to exit"),
         ])))
@@ -456,12 +493,15 @@ impl State {
         } else {
             use itertools::Itertools as _;
             let s = &results[selected].command;
-            s.char_indices()
-                .step_by(preview_width.into())
-                .map(|(i, _)| i)
-                .chain(Some(s.len()))
-                .tuple_windows()
-                .map(|(a, b)| &s[a..b])
+            s.split('\n')
+                .flat_map(|line| {
+                    line.char_indices()
+                        .step_by(preview_width.into())
+                        .map(|(i, _)| i)
+                        .chain(Some(line.len()))
+                        .tuple_windows()
+                        .map(|(a, b)| &line[a..b])
+                })
                 .join("\n")
         };
         let preview = if compact {
@@ -569,16 +609,19 @@ pub async fn history(
         search_mode: settings.search_mode,
         search: SearchState {
             input,
-            context,
-            filter_mode: if settings.shell_up_key_binding {
+            filter_mode: if settings.workspaces && context.git_root.is_some() {
+                FilterMode::Workspace
+            } else if settings.shell_up_key_binding {
                 settings
                     .filter_mode_shell_up_key_binding
                     .unwrap_or(settings.filter_mode)
             } else {
                 settings.filter_mode
             },
+            context,
         },
         engine: engines::engine(settings.search_mode),
+        results_len: 0,
     };
 
     let mut results = app.query_results(&mut db).await?;
@@ -596,7 +639,7 @@ pub async fn history(
             event_ready = event_ready => {
                 if event_ready?? {
                     loop {
-                        if let Some(i) = app.handle_input(settings, &event::read()?, results.len()) {
+                        if let Some(i) = app.handle_input(settings, &event::read()?) {
                             break 'render i;
                         }
                         if !event::poll(Duration::ZERO)? {
