@@ -1,7 +1,11 @@
 use clap::Subcommand;
 use eyre::{Result, WrapErr};
 
-use atuin_client::{database::Database, settings::Settings};
+use atuin_client::{
+    database::Database,
+    record::{store::Store, sync},
+    settings::Settings,
+};
 
 mod status;
 
@@ -37,9 +41,14 @@ pub enum Cmd {
 }
 
 impl Cmd {
-    pub async fn run(self, settings: Settings, db: &mut impl Database) -> Result<()> {
+    pub async fn run(
+        self,
+        settings: Settings,
+        db: &mut impl Database,
+        store: &mut (impl Store + Send + Sync),
+    ) -> Result<()> {
         match self {
-            Self::Sync { force } => run(&settings, force, db).await,
+            Self::Sync { force } => run(&settings, force, db, store).await,
             Self::Login(l) => l.run(&settings).await,
             Self::Logout => account::logout::run(&settings),
             Self::Register(r) => r.run(&settings).await,
@@ -62,12 +71,26 @@ impl Cmd {
     }
 }
 
-async fn run(settings: &Settings, force: bool, db: &mut impl Database) -> Result<()> {
+async fn run(
+    settings: &Settings,
+    force: bool,
+    db: &mut impl Database,
+    store: &mut (impl Store + Send + Sync),
+) -> Result<()> {
+    let (diff, remote_index) = sync::diff(settings, store).await?;
+    let operations = sync::operations(diff, store).await?;
+    let (uploaded, downloaded) =
+        sync::sync_remote(operations, &remote_index, store, settings).await?;
+
+    println!("{uploaded}/{downloaded} up/down to record store");
+
     atuin_client::sync::sync(settings, force, db).await?;
+
     println!(
-        "Sync complete! {} items in database, force: {}",
+        "Sync complete! {} items in history database, force: {}",
         db.history_count().await?,
         force
     );
+
     Ok(())
 }
