@@ -14,7 +14,7 @@ cat << EOF
 Magical shell history
 
 Atuin setup
-https://github.com/ellie/atuin
+https://github.com/atuinsh/atuin
 
 Please file an issue if you encounter any problems!
 
@@ -30,7 +30,7 @@ elif ! command -v sed &> /dev/null; then
     exit
 fi
 
-LATEST_RELEASE=$(curl -L -s -H 'Accept: application/json' https://github.com/ellie/atuin/releases/latest)
+LATEST_RELEASE=$(curl -L -s -H 'Accept: application/json' https://github.com/atuinsh/atuin/releases/latest)
 # Allow sed; sometimes it's more readable than ${variable//search/replace}
 # shellcheck disable=SC2001
 LATEST_VERSION=$(echo "$LATEST_RELEASE" | sed -e 's/.*"tag_name":"\([^"]*\)".*/\1/')
@@ -66,7 +66,7 @@ __atuin_install_arch(){
 __atuin_install_ubuntu(){
 	echo "Ubuntu detected"
 	# TODO: select correct AARCH too
-	ARTIFACT_URL="https://github.com/ellie/atuin/releases/download/$LATEST_VERSION/atuin_${LATEST_VERSION//v/}_amd64.deb"
+	ARTIFACT_URL="https://github.com/atuinsh/atuin/releases/download/$LATEST_VERSION/atuin_${LATEST_VERSION//v/}_amd64.deb"
 
 	TEMP_DEB="$(mktemp)".deb &&
 	curl -Lo "$TEMP_DEB" "$ARTIFACT_URL"
@@ -105,7 +105,6 @@ __atuin_install_mac(){
 	if command -v brew &> /dev/null
 	then
 		echo "Installing with brew"
-		brew tap ellie/atuin
 		brew install atuin
 	else
 		echo "Could not find brew, installing with Cargo"
@@ -150,7 +149,7 @@ __atuin_install_cargo(){
 
 __atuin_install_unsupported(){
 	echo "Unknown or unsupported OS"
-	echo "Please check the README at https://github.com/ellie/atuin for manual install instructions"
+	echo "Please check the README at https://github.com/atuinsh/atuin for manual install instructions"
 	echo "If you have any problems, please open an issue!"
 
 	while true; do
@@ -203,11 +202,8 @@ Thanks for installing Atuin! I really hope you like it.
 
 If you have any issues, please open an issue on GitHub or visit our Discord (https://discord.gg/dPhv2B3x)!
 
-Otherwise, Atuin is a hobby project - if you find it valuable, you can help us out!
+If you love Atuin, please give us a star on GitHub! It really helps ⭐️ https://github.com/atuinsh/atuin
 
-- ⭐️ Give us a star on GitHub (https://github.com/ellie/atuin)
-- 🚀 Contribute! We would love more regular contributors (https://github.com/ellie/atuin)
-- 🤑 Sponsor me! If you value the project + want to help keep the hosted sync server free (https://github.com/sponsors/ellie)
+Please run "atuin register" to get setup with sync, or "atuin login" if you already have an account
 
-~ Ellie 🐢💖
 EOF
